//+------------------------------------------------------------------+
//|                                           FXCorrelator.mq4       |
//|                                           Copyright  2009,      |
//|                                                                  |
//+------------------------------------------------------------------+
//
// Original version, including licensing terms can be found at:
// http://www.trade2win.com/boards/forex-strategies-systems/71854-correlation-trading-basic-ideas-strategies-64.html
//
// Change Log:
// 2010-06-01 (Till Gerken, till.gerken@gmail.com)
// - Added configurable symbol suffix for brokers that use
//   non-standard symbol names (EURUSDm, EURUSDgbp, etc.)
//
// 2010-04-09 (Till Gerken, till.gerken@gmail.com)
// - Added colored text labels to chart window to identify the currencies
// - Added text labels to the indicator buffers to identify a currency by
//   hovering above the price (see tooltip)
// - Added feature to turn off individual currencies
// - Added proper short name that contains indicator settings
//
// Unknown Date (NVP, et al)
// - Initial Version
//
#property copyright "FXCorrelator"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_color1 DeepSkyBlue //Blue
#property indicator_color2 Magenta //Red
#property indicator_color3 Goldenrod //Orange
#property indicator_color4 White //Gray
#property indicator_color5 Yellow
#property indicator_color6 Aqua //Turquoise
#property indicator_color7 Red //Brown
#property indicator_color8 Lime

#property indicator_level1 0.0

// Indicator Configuration
extern int PerAvr=200,Delta=3;
extern bool ShowEUR=true;
extern bool ShowGBP=true;
extern bool ShowAUD=true;
extern bool ShowCHF=true;
extern bool ShowJPY=true;
extern bool ShowNZD=true;
extern bool ShowCAD=true;
extern bool ShowUSD=true;
extern string SymbolSuffix="";

// index buffers for drawing
double Idx[],Idx1[],Idx2[],Idx3[],Idx4[],Idx5[],Idx6[],Idx7[];

// currency rates for calculation
double EUR,GBP,AUD,CHF,JPY,NZD,CAD,USD,A1,A2,A3,A4,A5,A6,A7;

// Currency names and colors
string Currencies[] = {"EUR","GBP","AUD","CHF","JPY","NZD","CAD","USD"};
int Colors[] = {indicator_color1, indicator_color2, indicator_color3, indicator_color4,
                indicator_color5, indicator_color6, indicator_color7, indicator_color8};

// house keeping
string ShortName;
bool ObjectsCreated = false;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   // set up indicator buffers
   for ( int i = 0; i < 8; i++ )
   {
      SetIndexStyle(i,DRAW_LINE,STYLE_SOLID,1);
      SetIndexLabel(i,Currencies[i]);
   }

   // silly MQL needs this part to be done manually
   SetIndexBuffer(0,Idx);
   SetIndexBuffer(1,Idx1);
   SetIndexBuffer(2,Idx2);
   SetIndexBuffer(3,Idx3);
   SetIndexBuffer(4,Idx4);
   SetIndexBuffer(5,Idx5);
   SetIndexBuffer(6,Idx6);
   SetIndexBuffer(7,Idx7);
   
   // Set name to be displayed
   ShortName = "FXCORR("+PerAvr+"/"+Delta+")";
   IndicatorShortName(ShortName);
   
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   // delete all objects we created
  ObjectsDeleteAll(WindowFind(ShortName));
  
  return(0);
}

//+------------------------------------------------------------------+
//| Create a single descriptive object at the given coordinates      |
//+------------------------------------------------------------------+
void CreateObject(string currency, int window, int x, int y, int col)
{
   int ret = ObjectCreate("obj_"+currency,OBJ_LABEL,window,0,0);
   ObjectSetText("obj_"+currency,currency,8);
   ObjectSet("obj_"+currency,OBJPROP_COLOR,col);
   ObjectSet("obj_"+currency,OBJPROP_XDISTANCE,x);
   ObjectSet("obj_"+currency,OBJPROP_YDISTANCE,y);
}

//+-------------------------------------------------------------------+
//| Create all text labels (colored names of the displayed currencies)|
//+-------------------------------------------------------------------+
void CreateObjects()
{
   // start and increment of X coordinates
   int xStart = 4;
   int xIncrement = 25;
   
   // start and increment of Y coordinates
   int yStart = 16;
   int yIncrement = 0;
   
   int window = WindowFind(ShortName);

   // create all labels
   for(int i = 0; i < 8; i++ )
   {
      CreateObject(Currencies[i], window, xStart, yStart, Colors[i]);
      xStart += xIncrement;
      yStart += yIncrement;
   }
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{  

   // only create objects here, window handle is not valid in init()  
   if ( !ObjectsCreated )
   {
      CreateObjects();
      ObjectsCreated = true;
   }
  
   int counted_bars=IndicatorCounted();
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   int limit=Bars-counted_bars;
   for(int i=limit; i>=0; i--)    
   {
      A1=(iMA("EURUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i)-iMA("EURUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta))/1; 
      A2=(iMA("GBPUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i)-iMA("GBPUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta))/1; 
      A3=(iMA("AUDUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i)-iMA("AUDUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta))/1; 
      A4=(iMA("USDCHF"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta)-iMA("USDCHF"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i))/1;
      A5=(iMA("USDJPY"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta)-iMA("USDJPY"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i))/100; 
      A6=(iMA("NZDUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i)-iMA("NZDUSD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta))/1;
      A7=(iMA("USDCAD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i+Delta)-iMA("USDCAD"+SymbolSuffix,0,PerAvr,0,MODE_LWMA,PRICE_CLOSE,i))/1;
      
      USD=-(A1+A2+A3+A4+A5+A6+A7)/7;
      EUR=A1-(A2+A3+A4+A5+A6+A7)/6;
      GBP=A2-(A1+A3+A4+A5+A6+A7)/6;
      AUD=A3-(A1+A2+A4+A5+A6+A7)/6;     
      CHF=A4-(A1+A2+A3+A5+A6+A7)/6;  
      JPY=A5-(A1+A2+A3+A4+A6+A7)/6;
      NZD=A6-(A1+A2+A3+A4+A5+A7)/6;
      CAD=A7-(A1+A2+A3+A4+A5+A6)/6;
             
      if ( ShowEUR )
         Idx[i] = EUR;
         
      if ( ShowGBP )
         Idx1[i] = GBP;
      
      if ( ShowAUD )
         Idx2[i] = AUD;
         
      if ( ShowCHF )
         Idx3[i] = CHF;
      
      if ( ShowJPY )
         Idx4[i] = JPY;
      
      if ( ShowNZD )
         Idx5[i] = NZD;
      
      if ( ShowCAD )
         Idx6[i] = CAD;
      
      if ( ShowUSD )
         Idx7[i] = USD;
   }

   return(0);
}
//+------------------------------------------------------------------+ 